local filteredIds 		= { --effect IDs that need to be ignored (thx @coolmodi!)
	[26298] = true, --cleansing ritual (which is called healing ritual in the API...) heal effect that is actually a buff without duration, yeah, wtf
	[64160] = true, --crystal fragments proc buff#2 whatever it even is for
	[66959] = true, --player vibration fx
}
local filteredIdsFade	= { --effect IDs that need to be ignored if they fire a fade event (thx @coolmodi!)
	[27090] = true, --Spear Shards cast/fly part, would cancel real ground aoe part
	[27167] = true, --Blazing Spear ^	
}

-- ------------------------
-- ADDON FUNCTIONS
-- ------------------------

function AuraMastery.OnEffectEvent(eventCode, changeType, effectSlot, effectName, unitTag, beginTime, endTime, stackCount, iconName, buffType, effectType, abilityType, statusEffectType, unitName, unitId, abilityId)	
	--d("Event triggered: "..changeType);
	if ("player " ~= unitTag and "Offline" == unitName) then return end -- unitTyg = source
	if (filteredIds[abilityId] or (filteredIdsFade[abilityId] and 2 == changeType)) then return end

	-- ability is being tracked
	if (AuraMastery.trackedEvents['EVENT_EFFECT_CHANGED'].abilityIds[abilityId]) then
		-- effect gained
		if (1 == changeType) then
--			d("Buff gained ("..abilityId.."): Begin ["..beginTime.."]; End ["..endTime.."]");
			if not AuraMastery.activeEffects[abilityId] then AuraMastery.activeEffects[abilityId] = {}; end
			table.insert(AuraMastery.activeEffects[abilityId], {['endTime'] = endTime, ['effectSlot'] = effectSlot});
		end

		-- effect faded
		if (2 == changeType) then
--			d("Buff faded ("..abilityId.."): Begin ["..beginTime.."]; End ["..endTime.."]");
			for i=1, #AuraMastery.activeEffects[abilityId] do
				if AuraMastery.activeEffects[abilityId][i]['effectSlot'] == effectSlot then
					AuraMastery.activeEffects[abilityId][i] = nil;
				end
			end
			if 0 == #AuraMastery.activeEffects[abilityId] then AuraMastery.activeEffects[abilityId] = nil; return; end
		end

		-- effect refreshed
		if (3 == changeType) then
			--d("Buff refreshed ("..abilityId.."): Begin ["..beginTime.."]; End ["..endTime.."]");
			for i=1, #AuraMastery.activeEffects[abilityId] do
				if AuraMastery.activeEffects[abilityId][i]['effectSlot'] == effectSlot then
					AuraMastery.activeEffects[abilityId][i]['endTime'] = endTime;
				end
			end
		end

		-- Force update to prevent display desync of eventTime <> addonUpdateFrequency
		AuraMastery:updateHandler();
	end
end

function AuraMastery.updateHandler()
	local n = #AuraMastery.loadedAuras;
	for i=1, n do
		AuraMastery:UpdateAura(AuraMastery.loadedAuras[i]);
	end
end

function AuraMastery:UpdateAura(auraName)
	local display, durationInfo = AuraMastery:TriggerAura(auraName);
	local criticalTime = 0;	-- will be updated below

	if (display) then
		-- Time Format
		if (nil ~= durationInfo) then
			if (durationInfo < AuraMastery.svars.auraData[auraName].critical) then 		
				criticalTime = AuraMastery.svars.auraData[auraName].critical;
				AuraMastery.aura[auraName].label:SetColor(1,0,0,1);
				AuraMastery.aura[auraName]:SetAlpha(1);
			else
				AuraMastery.aura[auraName].label:SetColor(.85,.85,.55,1);
				AuraMastery.aura[auraName]:SetAlpha(AuraMastery.svars.auraData[auraName].alpha);
			end
		end
	end

	local text = (nil ~= durationInfo and 0 <= durationInfo) and AuraMastery.FormatTime(durationInfo, criticalTime) or "";	-- Im Terniäroperator darf eine Methode nicht mit ":" aufgerufen werden, sondern nur mit dem object access operator "."
	AuraMastery.aura[auraName].label:SetText(text);	 -- Text muss zuerst gesetzt werden, weil sonst für einen winzigen Bruchteil einer Sekunde ein veralteter Wert (z.B. 0 oder sogar etwas < 0 angezeigt wird!)
	AuraMastery.aura[auraName]:SetHidden(not(display)); -- Logik muss umgekehrt werden, weil es SetHidden() und nicht SetVisible() ist!
end

function AuraMastery:TriggerAura(auraName)
	--if not(AuraMastery.svars.auraData[auraName]) then d("... ist nil!"); return; end
	--if not(AuraMastery.svars.auraData[auraName].triggers) then d("... kein Trigger gesetzt!"); return; end
	local bool;
	local sDuration;
	local triggers = AuraMastery.svars.auraData[auraName].triggers;
	for k, v in ipairs(triggers) do
		bool, duration = self:CheckCondition(v);
		if (nil == sDuration and 0 ~= duration) then sDuration = duration; end
		if false == bool then return bool, duration; end	-- if one condition failed, break and dont display
	end
	return bool, sDuration;
end

function AuraMastery:CheckCondition(triggerData)
	local returnValue = false;
	local timeLeft;
	local effectIndex = 1;
	if (1 == triggerData.triggerType) then
		local effectId = triggerData.spellId;
		timeLeft = AuraMastery.activeEffects[effectId] and AuraMastery.activeEffects[effectId][effectIndex]['endTime']-GetFrameTimeSeconds() or 0;

		-- check for all ability ranks here and set effectId appropriately
		if ("true" == triggerData.checkAll) then
			local lang = GetCVar("Language.2");
			local name = zo_strformat("<<z:1>>", GetAbilityName(triggerData.spellId));
			local endTime = -1;
			-- in case of multiple buffs of the same type with different ranks pick the one that has the highest duration
			if (self.abilityData[lang][name]) then
				for abilityRank, abilityId in pairs(self.abilityData[lang][name]) do
					if (AuraMastery.activeEffects[abilityId]) then
						local n = #self.activeEffects[abilityId];
						for i=1, n do
							if (self.activeEffects[abilityId][i]['endTime'] > endTime) then
								effectId   = abilityId;
								effectIndex = i;
								endTime  = self.activeEffects[abilityId][i]['endTime'];
								timeLeft = endTime-GetFrameTimeSeconds();
							end
						end
					end
				end
			end
		end

		if (self.activeEffects[effectId]) then
			if (">" == triggerData['operator']) then
				if timeLeft > triggerData['value'] then 
					returnValue = true;
				end
			elseif ("<" == triggerData['operator']) then
				if timeLeft < triggerData['value'] then
					returnValue = true;
				end
			elseif (">=" == triggerData['operator']) then
				if timeLeft >= triggerData['value'] then
					returnValue = true;
				end
			elseif ("<=" == triggerData['operator']) then
				if timeLeft <= triggerData['value'] then
					returnValue = true;
				end
			elseif ("==" == triggerData['operator']) then
				if timeLeft == triggerData['value'] then
					returnValue = true;
				end
			elseif ("~=" == triggerData['operator']) then
				if timeLeft ~= triggerData['value'] then
					returnValue = true;
				end
			end
		end
	end
	if ("true" == triggerData.invert) then returnValue = not(returnValue); end
	return returnValue, timeLeft;
end

function AuraMastery:GetDurationInfo(duration, effectIndex)
	-- if trigger contains an ability duration condition, get the shortest remaining buff
	if (AuraMastery.activeEffects[effectId]) then
		sDuration = AuraMastery.activeEffects[effectId][effectIndex]['endTime'];
	elseif (AuraMastery.activeEffects[effectId] and AuraMastery.activeEffects[effectId][effectIndex]['endTime'] < sDuration) then
		sDuration = AuraMastery.activeEffects[effectId][effectIndex]['endTime'];
	end
	return duration;
end

function AuraMastery:RemoveAuraData(auraName)
	self.svars.auraData[auraName] = nil;
end

AuraMastery.tTenths = '%.1f';
AuraMastery.tSec 	  = '%d';
AuraMastery.tMin 	  = '%dm';
AuraMastery.tHour	  = '%dh';
AuraMastery.tDay	  = '%dd';

-- ------------------------
-- HELPER FUNCTIONS
-- ------------------------
function AuraMastery.FormatTime(remaining, critical)
	if (remaining < critical) then -- 10ths of seconds
		return string.format(AuraMastery.tTenths, remaining)
	elseif (remaining < 60) then -- seconds
		return string.format(AuraMastery.tSec, remaining)
	elseif (remaining < 3600) then -- minutes
		return string.format(AuraMastery.tMin, math_floor(remaining / 60))
	elseif (remaining > 172800) then -- days (2+, revert to hours when between 24-48hrs)
		return string.format(AuraMastery.tDay, math_floor(remaining / 86400))
	else -- hours
		return string.format(AuraMastery.tHour, math_floor(remaining / 3600))
	end
end

function AuraMastery.TableIsEmpty(t)
	local next = next;
	if nil == next(t) then
   -- table is empty
		return true;
	end
end

function AuraMastery:GetNumElements(t)
  local count = 0
  for _ in pairs(t) do
		count = count + 1;
	end
	return count;
end

function AuraMastery:pairsByKeys (t, f)
	local a = {};
	for n in pairs(t) do
		table.insert(a, n);
	end
	table.sort(a, f)
	local i = 0 ;						-- iterator variable
	local iterator = function ()	-- iterator function
		i = i + 1;
		if (a[i] == nil) then
			return nil;
		else
			return a[i], t[a[i]];
		end
	end
	return iterator;
end

function AuraMastery:trim(s)
  return s:match'^()%s*$' and '' or s:match'^%s*(.*%S)'
end

function AuraMastery:GetToggled()
	-- returns all abilityIDs that match the names used as toggledAuras
	-- used to grab ALL the nessecary abilityIDs for the table after a patch changes things
	local data, names, saved = {}, {}, {}

	for k, v in pairs(toggledAuras) do
		names[GetAbilityName(k)] = true
	end

	for x = 1, 100000 do
		if (DoesAbilityExist(x) and names[GetAbilityName(x)] and GetAbilityDuration(x) == 0 and GetAbilityDescription(x) ~= '') then
			table.insert(data, {(GetAbilityName(x)), x, GetAbilityDescription(x)})
		end
	end

	table.sort(data, function(a, b)	return a[1] > b[1] end)

	for k, v in ipairs(data) do
		d(v[2] .. ' ' .. v[1] .. '      ' .. string.sub(v[3], 1, 30))
		table.insert(saved, v[2] .. '|' .. v[1]..'||' ..string.sub(v[3],1,30))
	end

	--SrendarrDB.toggled = saved
end

AuraMastery.iconPaths = {};

--[[

function AuraMastery:TriggerAura(auraName)
	if not(AuraMastery.svars.auraData[auraName]) then d("... ist nil!"); return; end
	if not(AuraMastery.svars.auraData[auraName].triggers) then d("... kein Trigger gesetzt!"); return; end
	local bool;
	local triggers = AuraMastery.svars.auraData[auraName].triggers;
	--local conditionStatement	= ""; -- string: complete if-statement containing all trigger-return-values and called later in zo_loadstring()
	local sDuration 				= nil;	-- "(buff/debuff x and buff/debuff y)" only remains true as long as the buff/debuff with the shorter duration to 0 remains up; this will contain the shortest debuff, @line 109
	for k, v in ipairs(triggers) do
		if false == self:CheckCondition(v) then
			bool = false;
		else
			bool = true;
		end

		-- Trigger is of type buff/debuff
		if (1 == v.triggerType) then
			local effectIndex = 1;
			local effectId    = v.spellId;

			-- Check for all ability ranks here and set effectId appropriately
			if v.checkAll then
				local lang = GetCVar("Language.2");
				local name = zo_strformat("<<z:1>>", GetAbilityName(v.spellId));
				local endTime = -1;
				

				-- in case of multiple buffs of the same type with different ranks pick the one with the highest duration
				if (self.abilityData[lang][name]) then
					for abilityRank, abilityId in pairs(self.abilityData[lang][name]) do
						if AuraMastery.activeEffects[abilityId] then
							local n = #self.activeEffects[abilityId];
							for i=1, n do
								if self.activeEffects[abilityId][i]['endTime'] > endTime then
									effectId   = abilityId;
									effectIndex = i;
									endTime    = self.activeEffects[abilityId][i]['endTime'];
								end
							end
						end
					end
				end
			end

			-- set trigger-logic
			local triggerCondition = "false";
			local triggerLogic     = (1==k and "") or " and ";	-- if 1st trigger then strip off logic operator
			
			-- set inverter
			local invert			= "true" == v.invert and "not " or "";
			local leftBracket		= v.invert and "(" or "";
			local rightBracket	= v.invert and ")" or "";

			local timeLeft = (AuraMastery.activeEffects[effectId] and AuraMastery.activeEffects[effectId][effectIndex]['endTime']-GetFrameTimeSeconds()) or nil; -- if a buff/debuff related to the current aura is up, save it's duration here; otherwise set it to nil
			local operator = v.operator;	-- logical operator to compare the remaining time of the buff/debuff to the user specified value
			local value		= v.value;		-- user specified: duration of buff/debuff related to this aura must be [operator specified above] than this specified value 
			
			if (nil ~= timeLeft) then		-- if no buff/debuff related to this trigger is up, the return value of this trigger will always be false, otherwise check for user specified duration conditions
				triggerCondition = timeLeft..operator..value;
			end

			conditionStatement = conditionStatement..triggerLogic.."("..invert..leftBracket..triggerCondition..rightBracket..")";	-- add current trigger's return-values to the complete if-statement

			-- if trigger contains an ability duration condition, get the shortest remaining buff
			if (nil == sDuration and AuraMastery.activeEffects[effectId]) then
				sDuration = AuraMastery.activeEffects[effectId][effectIndex]['endTime'];
			else
				if (AuraMastery.activeEffects[effectId] and AuraMastery.activeEffects[effectId][effectIndex]['endTime'] < sDuration) then
					sDuration = AuraMastery.activeEffects[effectId][effectIndex]['endTime'];
				end
			end
		end
	end
	if "" == conditionStatement then conditionStatement = "false"; end
	local cond = "if ("..conditionStatement..") then return true; else return false; end";
	--d(cond); -- show the executed condition string
	local bool = zo_loadstring(cond)();
	return bool, sDuration;
end

function AuraMastery:UpdateBar(auraName)
	local show, durationInfo = AuraMastery:TriggerAura(auraName);
	local criticalTime = 0;

	if (show) then
		-- Time Format
		if (nil ~= durationInfo and durationInfo > -1) then
			if (durationInfo-GetFrameTimeSeconds() < AuraMastery.svars.auraData[auraName].critical) then 		
				criticalTime = AuraMastery.svars.auraData[auraName].critical;
				AuraMastery.aura[auraName].label:SetColor(1,0,0,1);
				AuraMastery.aura[auraName]:SetAlpha(1);
			else
				AuraMastery.aura[auraName].label:SetColor(.85,.85,.55,1);
				AuraMastery.aura[auraName]:SetAlpha(AuraMastery.svars.auraData[auraName].alpha);
			end
		end
	end

	local text = (nil ~= durationInfo and AuraMastery.FormatTime(durationInfo-GetFrameTimeSeconds(), criticalTime)) or "";	-- Im Terniäroperator darf eine Methode nicht mit ":" aufgerufen werden, sondern nur mit dem object access operator "."
	AuraMastery.aura[auraName].label:SetText(text);	 -- Text muss zuerst gesetzt werden, weil sonst für einen winzigen Bruchteil einer Sekunde ein veralteter Wert (z.B. 0 oder sogar etwas < 0 angezeigt wird!)
	AuraMastery.aura[auraName]:SetHidden(not(show)); -- Logik muss umgekehrt werden, weil es SetHidden() und nicht SetVisible() ist!
end

function AuraMastery:TriggerAura(auraName)
	if not(AuraMastery.svars.auraData[auraName]) then d("... ist nil!"); return; end
	if not(AuraMastery.svars.auraData[auraName].triggers) then d("... kein Trigger gesetzt!"); return; end

	local triggers 				= AuraMastery.svars.auraData[auraName].triggers;
	local conditionStatement	= ""; -- string: complete if-statement containing all trigger-return-values and called later in zo_loadstring()
	local sDuration 				= nil;	-- "(buff/debuff x and buff/debuff y)" only remains true as long as the buff/debuff with the shorter duration to 0 remains up; this will contain the shortest debuff, @line 109
	for k, v in ipairs(triggers) do
		-- Trigger is of type buff/debuff
		if (1 == v.triggerType) then
			local effectIndex = 1;
			local effectId    = v.spellId;

			-- Check for all ability ranks here and set effectId appropriately
			if v.checkAll then
				local lang = GetCVar("Language.2");
				local name = zo_strformat("<<z:1>>", GetAbilityName(v.spellId));
				local endTime = -1;
				

				-- in case of multiple buffs of the same type with different ranks pick the one with the highest duration
				if (self.abilityData[lang][name]) then
					for abilityRank, abilityId in pairs(self.abilityData[lang][name]) do
						if AuraMastery.activeEffects[abilityId] then
							local n = #self.activeEffects[abilityId];
							for i=1, n do
								if self.activeEffects[abilityId][i]['endTime'] > endTime then
									effectId   = abilityId;
									effectIndex = i;
									endTime    = self.activeEffects[abilityId][i]['endTime'];
								end
							end
						end
					end
				end
			end

			-- set trigger-logic
			local triggerCondition = "false";
			local triggerLogic     = (1==k and "") or " and ";	-- if 1st trigger then strip off logic operator
			
			-- set inverter
			local invert			= "true" == v.invert and "not " or "";
			local leftBracket		= v.invert and "(" or "";
			local rightBracket	= v.invert and ")" or "";

			local timeLeft = (AuraMastery.activeEffects[effectId] and AuraMastery.activeEffects[effectId][effectIndex]['endTime']-GetFrameTimeSeconds()) or nil; -- if a buff/debuff related to the current aura is up, save it's duration here; otherwise set it to nil
			local operator = v.operator;	-- logical operator to compare the remaining time of the buff/debuff to the user specified value
			local value		= v.value;		-- user specified: duration of buff/debuff related to this aura must be [operator specified above] than this specified value 
			
			if (nil ~= timeLeft) then		-- if no buff/debuff related to this trigger is up, the return value of this trigger will always be false, otherwise check for user specified duration conditions
				triggerCondition = timeLeft..operator..value;
			end

			conditionStatement = conditionStatement..triggerLogic.."("..invert..leftBracket..triggerCondition..rightBracket..")";	-- add current trigger's return-values to the complete if-statement

			-- if trigger contains an ability duration condition, get the shortest remaining buff
			if (nil == sDuration and AuraMastery.activeEffects[effectId]) then
				sDuration = AuraMastery.activeEffects[effectId][effectIndex]['endTime'];
			else
				if (AuraMastery.activeEffects[effectId] and AuraMastery.activeEffects[effectId][effectIndex]['endTime'] < sDuration) then
					sDuration = AuraMastery.activeEffects[effectId][effectIndex]['endTime'];
				end
			end
		end
	end
	if "" == conditionStatement then conditionStatement = "false"; end
	local cond = "if ("..conditionStatement..") then return true; else return false; end";
	--d(cond); -- show the executed condition string
	local bool = zo_loadstring(cond)();
	return bool, sDuration;
end
--]]


--[[ Taken from Srendarr v2, great idea!

function AuraMastery:GetMajorEffects()
	local name
	self.majorEffects = {}
	for id = 1, 100000 do -- haven't seen any outside of these ids so compact search space a bit
		if (DoesAbilityExist(id) and GetAbilityDuration(id) > 0) then
			--name = string.sub(GetAbilityName(id), 1, 8)
			name = zo_strformat("<<z:1>>", GetAbilityName(id));
			if ("größere pflege" == name) then
--			if (name == 'größer') then
--				local abilityName = GetAbilityName(id);
--				local abilityIcon = GetAbilityIcon(id);
--				AuraMastery.iconPaths[id] = abilityIcon;
--				table.insert(SrendarrDB.majorBuffs, string.format('[%d] = %s, ', id, GetAbilityName(id)))
				table.insert(AuraMastery.majorEffects, {['abilityId'] = id, ['abilityName'] = GetAbilityName(id), ['abilityDuration'] = GetAbilityDuration(id), ['abilityIcon'] = GetAbilityIcon(id)})
			end
		end
	end
--	table.sort(self.iconPaths);
end

function AuraMastery:GetMinorEffects()
	local name
	SrendarrDB.minorBuffs = {}

	for id = 3000, 100000 do -- haven't seen any outside of these ids so compact search space a bit
		if (DoesAbilityExist(id) and GetAbilityDuration(id) > 0) then
			name = string.sub(GetAbilityName(id), 1, 6)

			if (name == 'Minor ') then
--				table.insert(SrendarrDB.minorBuffs, string.format('[%d] = %s, ', id, GetAbilityName(id)))
				table.insert(SrendarrDB.minorBuffs, string.format('%d|%s|%d|%s', id, GetAbilityName(id), GetAbilityDuration(id), GetAbilityIcon(id)))
			end
		end
	end
end


function AuraMastery:GetAurasByName(name)
	for x = 1, 100000 do
		if (DoesAbilityExist(x) and GetAbilityName(x) == name and GetAbilityDuration(x) > 0) then
			d('['..x ..'] '..GetAbilityName(x) .. '-' .. GetAbilityDuration(x) .. '-' .. GetAbilityDescription(x))
		end
	end
end

function AuraMastery:GetAuraInfo(idA, idB)
	d(string.format('[%d] %s (%ds) - %s', idA, GetAbilityName(idA), GetAbilityDuration(idA), GetAbilityDescription(idA)))
	d(string.format('[%d] %s (%ds) - %s', idB, GetAbilityName(idB), GetAbilityDuration(idB), GetAbilityDescription(idB)))
end

]]


--[[
------------------------
--PROFILE FUNCTIONS
------------------------

function AuraMastery:ConvertToNewProfileFormat()
	self.svars.numberBars 		= self.savedvars.numberBars					
	
	self.savedvars.numberBars	= nil					
	
	for name, barId in pairs(self.savedvars.buffNames) do
		self.svars.buffNames[name] = barId
	end
	self.savedvars.buffNames = nil
	
	for barId, data in pairs(self.savedvars.barData) do
		self.svars.barData[barId] = {}
		for key, value in pairs(data) do
			if value == type("table") then
				self.svars.barData[barId][key] = {}
				for k,v in pairs(value) do
					self.svars.barData[barId][key][k] = v
				end
			else
				self.svars.barData[barId][key] = value
			end			
		end
	end
	self.savedvars.barData = nil
end

function AuraMastery:LoadProfile(profileName)	
	local profId = -1
	for key, profName in pairs(self.savedvars.profiles.names) do
		if profName == profileName then profId = key end
	end	
	if profId == -1 then return end
	if profId == self.savedvars.activeProfile then d("Profile already active!") return end
	local oldNumberBars = self.svars.numberBars
	self.savedvars.activeProfile = profId
	self.svars = self.savedvars.profiles.profileData[self.savedvars.activeProfile]
	d("Loaded profile: "..profileName.." (ID:"..profId..")")
	if self.svars.numberBars > oldNumberBars then ReloadUI() end
	if self.svars.numberBars < oldNumberBars then 
		for i=self.svars.numberBars+1, oldNumberBars do
			if self.svars.barData[i] == nil then self.svars.barData[i] = {} end
			for k,v in pairs(self.defaultBarData) do
				if self.svars.barData[i][k] == nil then self.svars.barData[i][k] = v end
			end
			self.svars.barData[i].locked = true
		end
		self.svars.numberBars = oldNumberBars
		d("Profile has less bars then the previous, set to same number and filled additional active bars with default settings and locked them!")
	end
	self:SetControlSettings(oldNumberBars)
end

function AuraMastery:DeleteProfile(chosenProfile)
	local profId = -1
	for key, profName in pairs(self.savedvars.profiles.names) do
		if profName == chosenProfile then profId = key end
	end	

	if profId == self.savedvars.activeProfile then return end
 	
	table.remove(self.savedvars.profiles.names, profId)
	table.remove(self.savedvars.profiles.profileData, profId)
	
	if self.savedvars.activeProfile > profId then 
		self.savedvars.activeProfile = self.savedvars.activeProfile-1
		self.svars = self.savedvars.profiles.profileData[self.savedvars.activeProfile]
	end
end

function AuraMastery:RenameProfile(profileName, newName)
	for k, name in pairs(self.savedvars.profiles.names) do
		if name == profileName then self.savedvars.profiles.names[k] = newName end
	end
end

function AuraMastery:CreateProfile(name, templateProfile)
	if name == "" then return end
	table.insert(self.savedvars.profiles.names, name)
	if templateProfile == "Current profile" then templateProfile = self.savedvars.profiles.names[self.savedvars.activeProfile] end
	local newProfId = -1
	local templateId = -1
	for key, profName in pairs(self.savedvars.profiles.names) do
		if profName == name then newProfId = key end
		if profName == templateProfile then templateId = key end
	end	
	self.savedvars.profiles.profileData[newProfId] = {}	
	self:CopySettings(self.savedvars.profiles.profileData[newProfId], self.savedvars.profiles.profileData[templateId])
end

function AuraMastery:CopySettings(dest, source)
	for key, value in pairs(source) do
		if type(value) == "table" then
			dest[key] = {}
			self:CopySettings(dest[key], source[key])
		else
			dest[key] = value
		end
	end
end
--]]